@@ -1,5 +1,5 @@
 /*
- * $Id: com-config.c,v 1.6 2002/01/14 18:12:20 mt Exp $
+ * $Id: com-config.c,v 1.6.2.2 2002/04/04 10:00:07 mt Exp $
  *
  * Common functions for configuration reading
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-config.c,v 1.6 2002/01/14 18:12:20 mt Exp $";
+static char rcsid[] = "$Id: com-config.c,v 1.6.2.2 2002/04/04 10:00:07 mt Exp $";
 #endif
 
 #include <config.h>
@@ -283,6 +283,14 @@ void config_read(char *file, int dflg)
 				*data = '\0';
 			name = misc_strtrim(name + 1);
 
+			/*
+			** Do not accept empty sections or
+			** sections begining with a wildcard...
+			*/
+			if('\0' == name[0] || '*' == name[0]) {
+				misc_die(FL, "config_read: invalid section");
+			}
+
 			/*
 			** The global section is outstanding
 			*/
@@ -476,6 +484,43 @@ int config_sect(char *snam)
 }
 
 
+/* ------------------------------------------------------------ **
+**
+**	Function......:	config_sect_find
+**
+**	Parameters....:	snam		Section (NULL=global)
+**
+**	Return........:	pointer to the found the section or NULL
+**
+**	Purpose.......: find a config section by name; if snam
+**			is NULL the global section matches!
+**
+** ------------------------------------------------------------ */
+static SECTION* config_sect_find(char *snam)
+{
+	SECTION *sect;
+	char    *wild;
+
+	/*
+	** Find the relevant section
+	*/
+	for(sect = sechead; sect; sect = sect->next) {
+		if(sect->name && (wild = strchr(sect->name, '*'))) {
+#if defined(COMPILE_DEBUG)
+			debug(3, "config_sect_find: wildcard-sect='%.*s*'\n",
+				  wild - sect->name, sect->name);
+#endif
+			if (misc_strncaseequ(sect->name, snam,
+			                     wild - sect->name))
+				break;
+		} else {
+			if (misc_strcaseequ(sect->name, snam))
+				break;
+		}
+	}
+	return sect;
+}
+
 /* ------------------------------------------------------------ **
 **
 **	Function......:	config_int
@@ -510,10 +555,7 @@ int config_int(char *snam, char *name, int dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_int(NULL, name, dflt) : dflt);
 
@@ -579,10 +621,7 @@ int config_bool(char *snam, char *name, int dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_bool(NULL, name, dflt) : dflt);
 
@@ -657,10 +696,7 @@ char *config_str(char *snam, char *name, char *dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_str(NULL, name, dflt) : dflt);
 
@@ -721,10 +757,7 @@ u_int32_t config_addr(char *snam, char *name, u_int32_t dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_addr(NULL, name, dflt) : dflt);
 
@@ -790,10 +823,7 @@ u_int16_t config_port(char *snam, char *name, u_int16_t dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_port(NULL, name, dflt) : dflt);
 
@@ -859,10 +889,7 @@ uid_t config_uid(char *snam, char *name, uid_t dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_uid(NULL, name, dflt) : dflt);
 
@@ -940,10 +967,7 @@ gid_t config_gid(char *snam, char *name, gid_t dflt)
 	/*
 	** Find the relevant section
 	*/
-	for (sect = sechead; sect; sect = sect->next) {
-		if (misc_strcaseequ(snam, sect->name))
-			break;
-	}
+	sect = config_sect_find(snam);
 	if (sect == NULL)
 		return (snam ? config_gid(NULL, name, dflt) : dflt);
 
@@ -988,6 +1012,12 @@ gid_t config_gid(char *snam, char *name, gid_t dflt)
 
 /* ------------------------------------------------------------
  * $Log: com-config.c,v $
+ * Revision 1.6.2.2  2002/04/04 10:00:07  mt
+ * fixed bug done while last changes
+ *
+ * Revision 1.6.2.1  2002/01/28 01:55:58  mt
+ * implemented wildcard-section support
+ *
  * Revision 1.6  2002/01/14 18:12:20  mt
  * implemented config_dump function to dump in-memory config to a FILE stream
  *
