@@ -1,5 +1,5 @@
 /*
- * $Id: com-debug.c,v 1.4 2002/01/14 18:14:03 mt Exp $
+ * $Id: com-debug.c,v 1.4.2.1 2002/04/04 14:27:29 mt Exp $
  *
  * Common debugging functions
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-debug.c,v 1.4 2002/01/14 18:14:03 mt Exp $";
+static char rcsid[] = "$Id: com-debug.c,v 1.4.2.1 2002/04/04 14:27:29 mt Exp $";
 #endif
 
 #include <config.h>
@@ -43,6 +43,7 @@ static char rcsid[] = "$Id: com-debug.c,v 1.4 2002/01/14 18:14:03 mt Exp $";
 #  include <errno.h>
 #endif
 
+#include <sys/types.h>
 #if defined(HAVE_UNISTD_H)
 #  include <unistd.h>
 #endif
@@ -63,7 +64,6 @@ static char rcsid[] = "$Id: com-debug.c,v 1.4 2002/01/14 18:14:03 mt Exp $";
 #elif defined(HAVE_SYS_FCNTL_H)
 #  include <sys/fcntl.h>
 #endif
-
 #include <sys/stat.h>
 
 #include "com-debug.h"
@@ -166,11 +166,12 @@ void debug(int level, char *fmt, ...)
 	struct stat st;
 	time_t now;
 	struct tm *t;
+	mode_t omask;
 
 	/*
 	** Check if debug output is wanted
 	*/
-	if (level < 0 || level > dbg_lvl)
+	if (level <= 0 || level > dbg_lvl)
 		return;
 	if (!dbg_out || !*dbg_out || !fmt || !*fmt)
 		return;
@@ -178,8 +179,20 @@ void debug(int level, char *fmt, ...)
 	/*
 	** Check that the debug file has not been tampered with
 	*/
+	memset(&st, 0, sizeof(st));
 	if (lstat(dbg_out, &st) < 0) {
-		if ((fd = open(dbg_out, OPEN_NEW, 0666)) < 0)
+
+		/*
+		** Note: we adjust the umask temporary to force
+		** open to create a world-writeable debug file.
+		** See also notes in ftp-proxy(8) manual page
+		** and INSTALL file for compilation options.
+		*/
+		omask = umask(0);
+		fd = open(dbg_out, OPEN_NEW, 0666);
+		umask(omask);
+
+		if (fd < 0)
 			return;
 	} else {
 		if ((S_ISLNK(st.st_mode)) || (st.st_nlink > 1))
@@ -201,7 +214,7 @@ void debug(int level, char *fmt, ...)
 		va_end(aptr);
 		fprintf(fp, "\n");
 		fclose(fp);
-	}
+	} else close(fd);
 
 	errno = tmperr;			/* Restore errno	*/
 }
@@ -248,6 +261,9 @@ static void debug_cleanup(void)
 
 /* ------------------------------------------------------------
  * $Log: com-debug.c,v $
+ * Revision 1.4.2.1  2002/04/04 14:27:29  mt
+ * added umask(0) before opening new debug log file
+ *
  * Revision 1.4  2002/01/14 18:14:03  mt
  * fixed minor format bug
  *
