@@ -1,5 +1,5 @@
 /*
- * $Id: com-misc.c,v 1.8 2002/01/14 18:18:50 mt Exp $
+ * $Id: com-misc.c,v 1.8.2.1 2002/01/28 01:53:07 mt Exp $
  *
  * Common miscellaneous functions
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-misc.c,v 1.8 2002/01/14 18:18:50 mt Exp $";
+static char rcsid[] = "$Id: com-misc.c,v 1.8.2.1 2002/01/28 01:53:07 mt Exp $";
 #endif
 
 #include <config.h>
@@ -561,6 +561,46 @@ int misc_strcaseequ(const char *s1, const char *s2)
 }
 
 
+/* ------------------------------------------------------------ **
+**
+**	Function......:	misc_strnequ / misc_strncaseequ
+**
+**	Parameters....:	s1		First string to compare
+**			s2		Second string to compare
+**			n		number of characters in
+**					in s1 to compare
+**
+**	Return........:	1=strings are equal, 0=strings differ
+**
+**	Purpose.......: Check if two strings are equal. The
+**			strings could well be NULL pointers.
+**			strncasecmp ignores upper/lower case.
+**
+** ------------------------------------------------------------ */
+
+int misc_strnequ(const char *s1, const char *s2, size_t n)
+{
+	if (s1 == NULL && s2 == NULL)
+		return 1;
+	if (s1 == NULL && s2 != NULL)
+		return 0;
+	if (s1 != NULL && s2 == NULL)
+		return 0;
+	return (strncmp(s1, s2, n) == 0);
+}
+
+int misc_strncaseequ(const char *s1, const char *s2, size_t n)
+{
+	if (s1 == NULL && s2 == NULL)
+		return 1;
+	if (s1 == NULL && s2 != NULL)
+		return 0;
+	if (s1 != NULL && s2 == NULL)
+		return 0;
+	return (strncasecmp(s1, s2, n) == 0);
+}
+
+
 /* ------------------------------------------------------------ **
 **
 **	Function......:	misc_strncpy
@@ -744,6 +784,9 @@ int misc_rand (int lrng, int urng)
 
 /* ------------------------------------------------------------
  * $Log: com-misc.c,v $
+ * Revision 1.8.2.1  2002/01/28 01:53:07  mt
+ * implemented misc_strnequ misc_strncaseequ wrappers
+ *
  * Revision 1.8  2002/01/14 18:18:50  mt
  * implemented misc_chroot wrapper function
  * added checks in misc_uidgid if User/Group are set in config
