@@ -1,5 +1,5 @@
 /*
- * $Id: com-misc.h,v 1.4 2002/01/14 18:18:50 mt Exp $
+ * $Id: com-misc.h,v 1.4.2.1 2002/01/28 01:53:07 mt Exp $
  *
  * Header for common miscellaneous functions
  *
@@ -83,6 +83,8 @@ void  misc_pidfile(char *name);
 char *misc_strtrim   (char *s);
 int   misc_strequ    (const char *s1, const char *s2);
 int   misc_strcaseequ(const char *s1, const char *s2);
+int   misc_strnequ    (const char *s1, const char *s2, size_t n);
+int   misc_strncaseequ(const char *s1, const char *s2, size_t n);
 char *misc_strncpy   (char *s1, const char *s2, size_t len);
 
 int   misc_chroot (char *dir);
@@ -95,6 +97,9 @@ int   misc_rand (int lrng, int urng);
 
 /* ------------------------------------------------------------
  * $Log: com-misc.h,v $
+ * Revision 1.4.2.1  2002/01/28 01:53:07  mt
+ * implemented misc_strnequ misc_strncaseequ wrappers
+ *
  * Revision 1.4  2002/01/14 18:18:50  mt
  * implemented misc_chroot wrapper function
  * added checks in misc_uidgid if User/Group are set in config
