@@ -1,5 +1,5 @@
 /*
- * $Id: com-socket.c,v 1.6 2002/01/14 18:26:55 mt Exp $
+ * $Id: com-socket.c,v 1.6.2.2 2002/04/04 14:44:30 mt Exp $
  *
  * Common functions for TCP/IP sockets
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-socket.c,v 1.6 2002/01/14 18:26:55 mt Exp $";
+static char rcsid[] = "$Id: com-socket.c,v 1.6.2.2 2002/04/04 14:44:30 mt Exp $";
 #endif
 
 #include <config.h>
@@ -75,6 +75,9 @@ static char rcsid[] = "$Id: com-socket.c,v 1.6 2002/01/14 18:26:55 mt Exp $";
 
 #include <sys/socket.h>
 #include <netinet/in.h>
+#if defined(HAVE_NETINET_IN_SYSTM_H)
+#   include <netinet/in_systm.h>
+#endif
 #include <arpa/inet.h>
 #include <netdb.h>
 
@@ -87,10 +90,7 @@ static char rcsid[] = "$Id: com-socket.c,v 1.6 2002/01/14 18:26:55 mt Exp $";
 #if defined(HAVE_SYS_SOCKIO_H)
 #  include <sys/sockio.h>
 #endif
-#if defined(HAVE_NETINET_IN_SYSTM_H)
-#   include <netinet/in_systm.h>
-#endif
-#if defined (SOLARIS)
+#if defined(I_NREAD) && (defined(__sun__) || !defined(FIONREAD))
 #   define FIONREAD I_NREAD
 #endif
 
@@ -413,9 +413,10 @@ HLS *socket_init(int sock)
 	}
 
 	hls->kill = 0;
+	hls->ernr = 0;
 	hls->flag = 0;
 	hls->more = 0;
-	hls->ctyp = "???-????";
+	hls->ctyp = "HLS-TYPE";
 
 	hls->wbuf = NULL;
 	hls->rbuf = NULL;
@@ -993,6 +994,7 @@ int socket_exec(int timeout, int *close_flag)
 
 static void socket_ll_read(HLS *hls)
 {
+	static int retries = 0;
 	int len, cnt, nsock;
 	BUF *buf, *tmp;
 	struct sockaddr_in saddr;
@@ -1011,6 +1013,7 @@ static void socket_ll_read(HLS *hls)
 		nsock = accept(hls->sock,
 				(struct sockaddr *) &saddr, &len);
 		if (nsock < 0) {
+			hls->ernr = errno;
 			syslog_error("can't accept %s", hls->ctyp);
 			shutdown(hls->sock, 2);
 			close(hls->sock);
@@ -1039,28 +1042,54 @@ static void socket_ll_read(HLS *hls)
 	** Get the number of bytes waiting to be read
 	*/
 	len = 0;
-	if (ioctl(hls->sock, FIONREAD, &len) < 0) {
-		syslog_error("can't read from %d=%s",
-					hls->sock, hls->peer);
+	if( (cnt=ioctl(hls->sock, FIONREAD, &len)) < 0) {
+		hls->ernr = errno;
+		syslog_error("can't get num of bytes: %s %d=%s",
+					hls->ctyp, hls->sock, hls->peer);
 		return;
 	}
+#if defined(COMPILE_DEBUG)
+	debug(4, "ll_read: FIONREAD reported %d bytes for %s %d=%s",
+		len, hls->ctyp, hls->sock, hls->peer);
+#endif
 
 	/*
 	** Check if the socket has been closed
 	*/
 	if (len == 0) {
+#if defined(I_NREAD) && defined(__sun__)
+		/*
+		** solaris powers up select and returns "no data"
+		** (both, len == cnt == 0) in the middle of a data
+		** transfer; this was interpreted as EOF and has
+		** caused transfer aborts on bigger files.
+		** wait and retry before we assume this is a EOF.
+		*/
+		if((0 == cnt) && (++retries < MAX_RETRIES)) {
+			syslog_write(T_DBG,
+			      "zero bytes to read reported: %s %d=%s",
+			      hls->ctyp, hls->sock, hls->peer);
+			usleep(10000);
+			return;
+		}
+#endif
+
+		/*
+		** OK, EOF recived - should be cnt>0 and len == 0
+		** on solaris / I_NREAD according to streamio(7I)
+		*/
+#if defined(COMPILE_DEBUG)
+		debug(1, "closed: %s %d=%s, len=%d, cnt=%d",
+			hls->ctyp, hls->sock, hls->peer, len, cnt);
+#endif
 		close(hls->sock);
 		hls->sock = -1;
-#if defined(COMPILE_DEBUG)
-		debug(1, "closed: %s %d=%s",
-				hls->ctyp, hls->sock, hls->peer);
-#endif
 		return;
 	}
-#if defined(COMPILE_DEBUG)
-	debug(4, "ll_read: FIONREAD reported %d bytes for %s",
-		len, hls->ctyp);
-#endif
+	/*
+	** else reset retry counter on success
+	*/
+	retries = 0;
 
 	/*
 	** Limit the receive buffer sizes
@@ -1072,26 +1101,41 @@ static void socket_ll_read(HLS *hls)
 	** Now read the data that is waiting
 	*/
 	buf = (BUF *) misc_alloc(FL, sizeof(BUF) + len);
-	do
+	do {
+		errno = 0;
 		cnt = recv(hls->sock, buf->dat, len, 0);
-	while (cnt == -1 && errno == EINTR);
+	} while (cnt == -1 && EINTR == errno);
 
 	if (cnt != len) {
-		close(hls->sock);
-		hls->sock = -1;
-		misc_free(FL, buf);
-		syslog_error("can't ll_read: %s %d=%s",
-				hls->ctyp, hls->sock, hls->peer);
-		return;
+		if(cnt > 0) {
+			/*
+			** hmm... seems to be solaris, isn't? :-)
+			*/
+			syslog_write(T_DBG,
+			"recvd %d bytes while %d reported: %s %d=%s",
+			cnt, len, hls->ctyp, hls->sock, hls->peer);
+		} else {
+			/*
+			** report as error because we use FIONREAD
+			** above and handle EOF's (len = 0) there...
+			*/
+			hls->ernr = errno;
+			syslog_error("can't ll_read: %s %d=%s",
+			             hls->ctyp, hls->sock, hls->peer);
+			close(hls->sock);
+			hls->sock = -1;
+			misc_free(FL, buf);
+			return;
+		}
 	}
-	buf->len = len;
+	buf->len = cnt;
 	buf->cur = 0;
 	buf->flg = 0;
 
 	/*
 	** Update byte conter
 	*/
-	hls->rcnt += len;
+	hls->rcnt += cnt;
 
 	/*
 	** Chain the newly filled buffer
@@ -1108,7 +1152,7 @@ static void socket_ll_read(HLS *hls)
 #if defined(COMPILE_DEBUG)
 	debug(3, "ll_read %s %d=%s: %d/%d bytes",
 			hls->ctyp, hls->sock, hls->peer,
-			len, hls->rcnt);
+			cnt, hls->rcnt);
 #endif
 }
 
@@ -1147,10 +1191,11 @@ static void socket_ll_write(HLS *hls)
 		*/
 		if (cnt < 0) {
 			if (tot == 0) {
+				hls->ernr = errno;
+				syslog_error("can't ll_write: %s %d=%s",
+				             hls->ctyp, hls->sock, hls->peer);
 				close(hls->sock);
 				hls->sock = -1;
-				syslog_error("can't write to %d=%s",
-						hls->sock, hls->peer);
 				return;
 			}
 			break;	/* At least the first write was ok */
@@ -1281,7 +1326,6 @@ char *socket_msgline(char *fmt)
 			case '%':
 				tmp[0] = '%';
 				break;
-			default:
 		}
 		tmp[sizeof(tmp)-1] = '\0'; /* paranoia :-) */
 		j = strlen(tmp);
@@ -1956,41 +2000,51 @@ int socket_orgdst(HLS *phls, u_int32_t *addr, u_int16_t *port)
 	len = sizeof(dest);
 	memset(&dest, 0, len);
 	if(getsockopt(phls->sock, SOL_IP, SO_ORIGINAL_DST, &dest, &len) < 0) {
-		/*
-		** ENOPROTOOPT: 2.2 kernel ... no iptables support
-		** ENOENT     : 2.4 kernel without iptables support
-		*/
-		if(ENOPROTOOPT == errno || ENOENT == errno) {
-			if(ENOENT == errno)
-			syslog_write(T_DBG,
+		switch(errno) {
+		case ENOPROTOOPT:
+			/*
+			** no iptables support / 2.2 kernel
+			** ==> use getsockname dst bellow
+			*/
+		break;
+		case ENOENT:
+			/*
+			** 2.4 kernel without iptables support
+			** ==> getsockname does not work here
+			*/
+			syslog_write(T_WRN,
 			"iptables not supported or ipchains support active");
-		} else {
-			syslog_error(
-			"can't get iptables transparent proxy destination");
 			return -1;
+		break;
+		default:
+			syslog_error(
+			"can't get iptables transparent destination");
+			return -1;
+		break;
 		}
 	} else {
-#if defined(COMPILE_DEBUG)
-		debug(2, "iptables transparent proxy destination: %s:%d",
-			 socket_addr2str(ntohl(dest.sin_addr.s_addr)),
-			 ntohs(dest.sin_port));
-#endif
 		if((name.sin_port == dest.sin_port) &&
 		   (name.sin_addr.s_addr == dest.sin_addr.s_addr)) {
 			syslog_write(T_DBG,
-			       "iptables proxy destination %s:%d is local",
-			       socket_addr2str(ntohl(dest.sin_addr.s_addr)),
-			       ntohs(dest.sin_port));
+			      "iptables transparent destination %s:%d is local",
+			      socket_addr2str(ntohl(dest.sin_addr.s_addr)),
+			      ntohs(dest.sin_port));
 			return -1;
 		}
+
+		syslog_write(T_DBG,
+			"iptables transparent destination: %s:%d",
+			socket_addr2str(ntohl(dest.sin_addr.s_addr)),
+			ntohs(dest.sin_port));
+
 		*addr = dest.sin_addr.s_addr;
 		*port = dest.sin_port;
+
 		return 0;
 	}
 #endif
 
 #if defined(HAVE_NETINET_IP_NAT_H) && defined(SIOCGNATL)
-
 	/*
 	** BSD ipnat table lookup
 	*/
@@ -2020,29 +2074,28 @@ int socket_orgdst(HLS *phls, u_int32_t *addr, u_int16_t *port)
 
 	if(rc < 0) {
 		if(errno != ESRCH) {
-			syslog_error("can't get ipnat transparent "
-			             "proxy destination");
+			syslog_error("can't get ipnat transparent destination");
 		}
 		return -1;
 	}
 
-#if defined(COMPILE_DEBUG)
-	debug(2, "ipnat transparent proxy destination: %s:%d",
-		       socket_addr2str(ntohl(natlook.nl_realip.s_addr)),
-		       ntohs(natlook.nl_realport));
-#endif
-
 	if((natlook.nl_realport == name.sin_port) &&
 	   (natlook.nl_realip.s_addr == name.sin_addr.s_addr))
 	{
 		syslog_write(T_DBG,
-		       "ipnat proxy destination %s:%d is local",
+		       "ipnat transparent destination %s:%d is local",
 		       socket_addr2str(ntohl(natlook.nl_realip.s_addr)),
 		       ntohs(natlook.nl_realport));
 		return -1;
 	} else {
+		syslog_write(T_DBG,
+			"ipnat transparent destination: %s:%d",
+			socket_addr2str(ntohl(natlook.nl_realip.s_addr)),
+			ntohs(natlook.nl_realport));
+
 		*addr = natlook.nl_realip.s_addr;
 		*port = natlook.nl_realport;
+
 		return 0;
 	}
 
@@ -2051,16 +2104,13 @@ int socket_orgdst(HLS *phls, u_int32_t *addr, u_int16_t *port)
 	/*
 	** IP-Chains uses getsockname, as "transparent address"
 	*/
-
-#if defined(COMPILE_DEBUG)
-		debug(2, "ipchains transparent proxy destination: %s:%d",
-			 socket_addr2str(ntohl(name.sin_addr.s_addr)),
-			 ntohs(name.sin_port));
-#endif
+	syslog_write(T_DBG,
+			"ipchains transparent destination: %s:%d",
+			socket_addr2str(ntohl(name.sin_addr.s_addr)),
+			ntohs(name.sin_port));
 
 	*addr = name.sin_addr.s_addr;
 	*port = name.sin_port;
-
 #endif
 	return 0;
 }
@@ -2135,6 +2185,15 @@ int getfqdomainname(char *fqdomain, size_t n)
 
 /* ------------------------------------------------------------
  * $Log: com-socket.c,v $
+ * Revision 1.6.2.2  2002/04/04 14:44:30  mt
+ * added waiting and retrying on "no data" but no EOF in I_NREAD
+ * added check for buffer len difference on ioctl and recv
+ * added remembering of i/o failures in hls->ernr for loging
+ * added and improved transparent proxy log messages
+ *
+ * Revision 1.6.2.1  2002/01/28 01:51:21  mt
+ * replaced question marks sequences that may be misinterpreted as trigraphs
+ *
  * Revision 1.6  2002/01/14 18:26:55  mt
  * implemented socket_orgdst to read transparent proxying destinations
  * implemented a MaxRecvBufSize option limiting max recv buffer size
