@@ -1,5 +1,5 @@
 /*
- * $Id: com-socket.h,v 1.4 2002/01/14 18:26:55 mt Exp $
+ * $Id: com-socket.h,v 1.4.2.1 2002/04/04 14:33:17 mt Exp $
  *
  * Header for common functions for TCP/IP sockets
  *
@@ -87,6 +87,7 @@ typedef struct hls_t {
 	struct hls_t *next;	/* Next one in the chain	*/
 	int       sock;		/* The corresponding socket	*/
 	int       kill;		/* 1=kill socket after send	*/
+	int       ernr;		/* socket i/o error number	*/
 	int       flag;		/* Flag for send() (e.g. OOB)	*/
 	int       more;		/* 1=read more to complete line	*/
 	u_int32_t addr;		/* Peer's address (host order)	*/
@@ -150,6 +151,9 @@ int        getfqdomainname(char *fqdomain, size_t n);
 
 /* ------------------------------------------------------------
  * $Log: com-socket.h,v $
+ * Revision 1.4.2.1  2002/04/04 14:33:17  mt
+ * added ernr flag in hls needed to remember failures
+ *
  * Revision 1.4  2002/01/14 18:26:55  mt
  * implemented socket_orgdst to read transparent proxying destinations
  * implemented a MaxRecvBufSize option limiting max recv buffer size
