@@ -1,5 +1,5 @@
 /*
- * $Id: com-syslog.c,v 1.5 2002/01/14 18:30:15 mt Exp $
+ * $Id: com-syslog.c,v 1.5.2.2 2002/04/04 14:28:53 mt Exp $
  *
  * Common file/syslog logging functions
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: com-syslog.c,v 1.5 2002/01/14 18:30:15 mt Exp $";
+static char rcsid[] = "$Id: com-syslog.c,v 1.5.2.2 2002/04/04 14:28:53 mt Exp $";
 #endif
 
 #include <config.h>
@@ -392,7 +392,7 @@ void syslog_write(int level, char *fmt, ...)
 				break;
 
 		default:	loglvl = LOG_CRIT;
-				logstr = "????-???";
+				logstr = "TECH-FLT";
 				dbglvl = 1;
 	}
 
@@ -555,10 +555,10 @@ int  syslog_rename(char *new_name, char *log_name, size_t len)
 		t->tm_hour, t->tm_min, t->tm_sec);
 #endif
 
-	if( stat(log_name, &st))
+	if( lstat(log_name, &st))
 		return 1;
 
-	if( !stat(new_name, &st)) {
+	if( !lstat(new_name, &st)) {
 		if( !(S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))) {
 			return -1;
 		}
@@ -663,6 +663,12 @@ void syslog_close(void)
 
 /* ------------------------------------------------------------
  * $Log: com-syslog.c,v $
+ * Revision 1.5.2.2  2002/04/04 14:28:53  mt
+ * replaced stat with lstat for rename check in log rotation
+ *
+ * Revision 1.5.2.1  2002/01/28 01:51:21  mt
+ * replaced question marks sequences that may be misinterpreted as trigraphs
+ *
  * Revision 1.5  2002/01/14 18:30:15  mt
  * implemented syslog_stderr function to redirect log to stderr
  * added LogLevel option handling allowing to set the maximal level
