@@ -1,5 +1,5 @@
 /*
- * $Id: ftp-client.c,v 1.8 2002/01/14 19:35:44 mt Exp $
+ * $Id: ftp-client.c,v 1.8.2.1 2002/04/04 14:26:11 mt Exp $
  *
  * Functions for the FTP Proxy client handling
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: ftp-client.c,v 1.8 2002/01/14 19:35:44 mt Exp $";
+static char rcsid[] = "$Id: ftp-client.c,v 1.8.2.1 2002/04/04 14:26:11 mt Exp $";
 #endif
 
 #include <config.h>
@@ -131,7 +131,7 @@ static RETSIGTYPE client_signal(int signo)
 
 void client_run(void)
 {
-	int  sock, need, diff, rw_prio;
+	int  sock, need, diff;
 	char str[MAX_PATH_SIZE * 2];
 	char *p, *q;
 	FILE *fp;
@@ -209,12 +209,6 @@ void client_run(void)
 	client_respond(220,
 		config_str(NULL, "WelcomeMessage", NULL), str);
 
-	/*
-	** Set the "read only if nothing to write there" flag
-	** for data transfers if a read bufffer limit is set.
-	*/
-	rw_prio = config_int(NULL, "MaxRecvBufSize", 0);
-
 	/*
 	** Enter the client mainloop
 	*/
@@ -238,9 +232,10 @@ void client_run(void)
 			need = 1;
 
 		/*
-		** use higher priority to writes
+		** use higher priority to writes;
+		** read only if nothing to write...
 		*/
-		if(rw_prio > 0 && ctx.srv_data && ctx.cli_data) {
+		if(ctx.srv_data && ctx.cli_data) {
 			if(ctx.srv_data->wbuf) {
 				ctx.cli_data->more = -1;
 			} else {
@@ -303,17 +298,44 @@ void client_run(void)
 			/*
 			** If we have an outstanding server reply
 			** (e.g. 226 Transfer complete), send it.
-			** If we have not, it will come sooner or
-			** later. Then just send it to the client
-			** at the time it arrives.
 			*/
 			if (ctx.xfer_rep[0] != '\0') {
 				socket_printf(ctx.cli_ctrl,
 					"%s\r\n", ctx.xfer_rep);
 				memset(ctx.xfer_rep, 0,
 					sizeof(ctx.xfer_rep));
-			} else if (ctx.expect == EXP_XFER)
-				ctx.expect = EXP_PTHR;
+			} else {
+				/*
+				** if an error occured and srv_data xfer
+				** is still running, try to ABORt it
+				**     else
+				** If we have outstanding server reply,
+				** it will come sooner or later. Then just
+				** mark it have to be passed to the client
+				** at the time it arrives.
+				*/
+				if( 0 != ctx.cli_data->ernr &&
+				   NULL != ctx.srv_data &&
+				   -1 != ctx.srv_data->sock)
+				{
+#if defined(MSG_OOB)
+					char str[4] = { IAC, IP, IAC, 0 };
+
+					socket_flag(ctx.srv_ctrl, MSG_OOB);
+					socket_write(ctx.srv_ctrl, str, 3);
+					socket_flag(ctx.srv_ctrl, 0);
+					str[0] = DM;
+					socket_write(ctx.srv_ctrl, str, 1);
+#endif
+					socket_printf(ctx.srv_ctrl,"ABOR\r\n");
+					client_respond(426, NULL,
+					"Connection closed; transfer aborted");
+					ctx.expect = EXP_ABOR;
+				} else {
+					if(ctx.expect == EXP_XFER)
+						ctx.expect = EXP_PTHR;
+				}
+			}
 
 			/*
 			** Good time for statistics and data reset
@@ -324,17 +346,21 @@ void client_run(void)
 			if (diff < 1)
 				diff = 1;
 
+			/*
+			** print our current statistic
+			*/
 			syslog_write(U_INF,
-				"Transfer for %s: %s '%s' %s %u/%d byte/sec",
-				ctx.cli_ctrl->peer, ctx.xfer_cmd,
-				ctx.xfer_arg,
+				"Transfer for %s %s: %s '%s' %s %u/%d byte/sec",
+				ctx.cli_ctrl->peer,
+				ctx.cli_data->ernr ?  "failed" : "completed",
+				ctx.xfer_cmd, ctx.xfer_arg,
 				ctx.cli_data->rcnt ? "sent" : "read",
 				ctx.cli_data->rcnt ? ctx.cli_data->rcnt
 				                   : ctx.cli_data->wcnt,
 				diff);
 
 			/*
-			** update statistics data
+			** update session statistics data
 			*/
 			if(ctx.cli_data->rcnt)
 				ctx.xfer_rsec += diff;
@@ -344,7 +370,7 @@ void client_run(void)
 			ctx.xfer_wcnt += ctx.cli_data->wcnt;
 
 			/*
-			** data transfer reset
+			** reset data transfer state
 			*/
 			client_data_reset(MOD_RESET);
 
@@ -367,10 +393,15 @@ void client_run(void)
 			debug(3, "about to destroy Srv-Data");
 #endif
 			/*
-			** Doom the corresponding client socket _except_
-			** FailResetsPasv=no and we expect PASV response
+			** Doom the corresponding client socket if an
+			** error occured, FailResetsPasv=yes or we
+			** expect other response than PASV (Netscape!)
 			*/
 			if(ctx.cli_data != NULL) {
+				if(0 != ctx.srv_data->ernr) {
+					ctx.cli_data->ernr = -1;
+					ctx.cli_data->kill =  1;
+				}
 				if(config_bool(NULL,"FailResetsPasv", 0)) {
 					ctx.cli_data->kill = 1;
 				} else if(ctx.expect != EXP_PASV) {
@@ -643,6 +674,8 @@ static void client_srv_ctrl_read(char *str)
 	if (str == NULL)		/* Basic sanity check	*/
 		return;
 
+	syslog_write(T_DBG, "from Server-PI (%d): '%.512s'",
+	             ctx.srv_ctrl->sock, str);
 #if defined(COMPILE_DEBUG)
 	debug(1, "from Server-PI (%d): '%.512s'",
 					ctx.srv_ctrl->sock, str);
@@ -1059,6 +1092,9 @@ void client_data_reset(int mode)
 
 /* ------------------------------------------------------------
  * $Log: ftp-client.c,v $
+ * Revision 1.8.2.1  2002/04/04 14:26:11  mt
+ * added failed/completed transfer log message status
+ *
  * Revision 1.8  2002/01/14 19:35:44  mt
  * implemented workarround for Netscape (4.x) directory symlink handling
  * implemented a MaxRecvBufSize option limiting max recv buffer size
