@@ -1,5 +1,5 @@
 /*
- * $Id: ftp-cmds.c,v 1.9 2002/01/14 19:39:30 mt Exp $
+ * $Id: ftp-cmds.c,v 1.9.2.1 2002/04/04 14:23:32 mt Exp $
  *
  * FTP Proxy command handling
  *
@@ -30,7 +30,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: ftp-cmds.c,v 1.9 2002/01/14 19:39:30 mt Exp $";
+static char rcsid[] = "$Id: ftp-cmds.c,v 1.9.2.1 2002/04/04 14:23:32 mt Exp $";
 #endif
 
 #include <config.h>
@@ -398,6 +398,9 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 	/*
 	** Check for permission and existence of "transparent proxy"
 	** magic destination address and port from the client socket
+	**
+	** "fall through" on error and proceed to check magic user
+	** or use DestinationAddress from config...
 	*/
 	ctx->magic_addr = INADDR_ANY;
 	ctx->magic_port = INPORT_ANY;
@@ -421,7 +424,7 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 			switch( rc) {
 			case -1:
 				syslog_write(T_ERR,
-				"check of transparent proxy dest failed");
+				"check of transparent destination failed");
 			break;
 			case 0:
 				ctx->magic_addr = ntohl(addr);
@@ -432,11 +435,14 @@ static void cmds_user(CONTEXT *ctx, char *arg)
 				ctx->magic_port, ctx->cli_ctrl->peer);
 			break;
 			default:
-				syslog_write(T_DBG,
-				"requested transparent proxy dest %s is local",
-				socket_addr2str(ntohl(addr)));
+			syslog_write(T_DBG,
+			  "requested transparent destination %s is local",
+			  socket_addr2str(ntohl(addr)));
 			break;
 			}
+		} else {
+			syslog_write(T_DBG,
+			"no transparent proxy destination found");
 		}
 	}
 
@@ -1357,6 +1363,9 @@ char *cmds_reg_exec(void *regex, char *str)
 
 /* ------------------------------------------------------------
  * $Log: ftp-cmds.c,v $
+ * Revision 1.9.2.1  2002/04/04 14:23:32  mt
+ * improved transparent proxy log messages
+ *
  * Revision 1.9  2002/01/14 19:39:30  mt
  * implemented workarround for Netscape (4.x) directory symlink handling
  * changed to socket_orgdst usage to get ransparent proxy destination
